#ifndef __TaskThread__
#define __TaskThread__

#include <iostream>
#include <queue>
#include <pthread.h>
#include <memory>

#include "Runnable.hpp"
#include "CqEventData.hpp"
#include "MessageFormats.hpp"
#include "Logger.hpp"
#include "ConcurrentQueue.hpp"
#include "RdmaRepEndpoint.hpp"
#include "RdmaSalEndpoint.hpp"

class TaskThread
{

private:
    ConcurrentQueue *_taskQueue;
    bool _stop{false};
    int _id;
    pthread_t thread;
    std::unordered_map<uint32_t, RdmaRepEndpoint *> *_clientRepMap;
    std::unordered_map<uint32_t, RdmaRepEndpoint *> *_serverRepMap;
    std::unordered_map<uint32_t, RdmaSalEndpoint *> *_salMap;
    std::unordered_map<uint32_t, std::shared_ptr<CQEventData>> &_cqEvents;
    std::mutex &_cqEventsMutex;

public:
    TaskThread(int id, int cpu, ConcurrentQueue *,
               std::unordered_map<uint32_t, RdmaRepEndpoint *> *clientRepMap,
               std::unordered_map<uint32_t, RdmaRepEndpoint *> *serverRepMap,
               std::unordered_map<uint32_t, RdmaSalEndpoint *> *salMap,
               std::unordered_map<uint32_t, std::shared_ptr<CQEventData>> &cqEvents,
               std::mutex &_cqEventsMutex);

    void replicateSalRequest(char *salRequest, uint32_t size);
    void sendInvalidation(char *salRequest);
    static void *run(void *object);
    void stop();
    void processSalEvent(RdmaSalEndpoint *ep, struct ibv_wc *data);
    void processRepEvent(RdmaRepEndpoint *ep, struct ibv_wc *data);
    void processReplicationDone(uint32_t reqid, MessageType type, bool localWrite);
    ~TaskThread();
};

#endif